/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.anweisung;

import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * die Repräsentation einer Fehlerprüfungsanweisung ohne Fehlerprüfung.<br>
 * Die Anweisung wertet den enthaltenen Ausdruck aus. Wird bei der Auswertung
 * des Ausdrucks eine Fehlerausnahme augelöst, wird dieser ignoriert. Die
 * Ausführung des Skripts wird in jedem Fall fortgesetzt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class OhneFehlerPruefungAnweisung extends FehlerPruefungsAnweisung {

	private static final Debug LOGGER = Debug.getLogger();

	/**
	 * Konstruktor.
	 *
	 * @param zeile
	 *            die Nummer der Zeile, in der die Anweisung definiert wurde
	 * @param skriptname
	 *            der Name des Skripts
	 * @param ausdruck
	 *            der auszuwertende Ausdruck
	 */
	public OhneFehlerPruefungAnweisung(final int zeile, final String skriptname, final Ausdruck ausdruck) {
		super(zeile, skriptname, ausdruck);
	}

	/**
	 * Die Funktion wertet den der Anweisung zugeordneten Ausdruck aus.<br>
	 * Wird bei der Auswertung eine Ausnahme erzeugt, wird diese ignoriert. Das
	 * Ergebnis der Auswertung des Ausdrucks oder der Wert "undefiniert" wird
	 * zurückgeliefert.<br>
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck#interpret(de.bsvrz.sys.funclib.bitctrl.interpreter.Kontext)
	 */
	@Override
	public Object interpret(final Kontext kontext) {
		Object result = null;
		try {
			result = getAusdruck().interpret(kontext);
		} catch (final InterpreterException e) {
			OhneFehlerPruefungAnweisung.LOGGER.warning(e.getLocalizedMessage());
			// Fehler werden ignoriert
		}
		if (result == null) {
			result = NichtWert.NICHTWERT;
		}
		return result;
	}
}
