/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.container;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ContainerZugriffsFehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;

/**
 * Testklasse zur Prüfung der Funktionen der UdaContainer-Klasse Feld.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class FeldTest {

	/**
	 * das Feld mit dem die Tests ausgeführt werden.
	 */
	private Feld testFeld;

	/**
	 * füllt das Feld mit Daten.
	 */
	@Before
	public void setupTestFeld() {
		testFeld = new Feld(1);
		testFeld.anhaengen(-1, 1);
		testFeld.anhaengen(-1, 2);
		testFeld.anhaengen(-1, 3);
		testFeld.anhaengen(-1, 4);
	}

	/**
	 * testet den Konstruktor ohne Argumente.
	 */
	@Test
	public void testFeld() {
		Assert.assertEquals(0, new Feld().getGroesse());
	}

	/**
	 * testet den Konstruktor mit Angabe der initialen Feldgröße.
	 */
	@Test
	public void testFeldNumber() {
		Feld feld = new Feld(100);
		Assert.assertEquals(100, feld.getGroesse());
		for (int idx = 1; idx <= 100; idx++) {
			Assert.assertEquals(NichtWert.NICHTWERT, feld.getContainerElement(idx));
		}

		feld = new Feld(0);
		Assert.assertEquals(0, feld.getGroesse());

		try {
			new Feld(-100);
			Assert.fail("Feld mit negativer Initialgröße angelegt");
		} catch (final ArgumentFehler e) {
			// negative Initialgröße ist nicht zulässig
		}
	}

	/**
	 * testet den Konstruktor mit Angabe eines initialen Containers.
	 */
	@Test
	public void testFeldUdaContainer() {
		final Feld feld = new Feld(testFeld);
		Assert.assertEquals(5, feld.getGroesse());
		Assert.assertEquals(NichtWert.NICHTWERT, feld.getContainerElement(1));
		Assert.assertEquals(1, ((Number) feld.getContainerElement(2)).intValue());
		Assert.assertEquals(2, ((Number) feld.getContainerElement(3)).intValue());
		Assert.assertEquals(3, ((Number) feld.getContainerElement(4)).intValue());
		Assert.assertEquals(4, ((Number) feld.getContainerElement(5)).intValue());
	}

	/**
	 * testet die Einfügefunktion für ein Feld.
	 */
	@Test
	public void testEinfuegenIndex() {
		Feld feld = new Feld(testFeld);
		feld.einfuegen(1, 999);
		Assert.assertEquals("[999,undefiniert,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(2, 999);
		Assert.assertEquals("[undefiniert,999,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(3, 999);
		Assert.assertEquals("[undefiniert,1,999,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(4, 999);
		Assert.assertEquals("[undefiniert,1,2,999,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(5, 999);
		Assert.assertEquals("[undefiniert,1,2,3,999,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(6, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(150, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(0, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(-1, 999);
		Assert.assertEquals("[undefiniert,1,2,3,999,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(-2, 999);
		Assert.assertEquals("[undefiniert,1,2,999,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(-3, 999);
		Assert.assertEquals("[undefiniert,1,999,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(-4, 999);
		Assert.assertEquals("[undefiniert,999,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(-5, 999);
		Assert.assertEquals("[999,undefiniert,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(-6, 999);
		Assert.assertEquals("[999,undefiniert,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.einfuegen(-150, 999);
		Assert.assertEquals("[999,undefiniert,1,2,3,4]", feld.toString());
	}

	/**
	 * testet die Einfügefunktion über eine Schlüssel.
	 */
	@Test
	public void testElementEinfuegenSchluessel() {
		try {
			testFeld.einfuegen("Heinz", 999);
			Assert.fail("Einfügen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel eingefügt werden
		}
		try {
			testFeld.einfuegen("1", 999);
			Assert.fail("Einfügen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel eingefügt werden
		}
		try {
			testFeld.einfuegen("", 999);
			Assert.fail("Einfügen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel eingefügt werden
		}
		try {
			testFeld.einfuegen(null, 999);
			Assert.fail("Einfügen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel eingefügt werden
		}
	}

	/**
	 * testet die Anhängefunktion für ein Feld.
	 */
	@Test
	public void testAnhaengenIndex() {
		Feld feld = new Feld(testFeld);
		feld.anhaengen(1, 999);
		Assert.assertEquals("[undefiniert,999,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(2, 999);
		Assert.assertEquals("[undefiniert,1,999,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(3, 999);
		Assert.assertEquals("[undefiniert,1,2,999,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(4, 999);
		Assert.assertEquals("[undefiniert,1,2,3,999,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(5, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(6, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(150, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(0, 999);
		Assert.assertEquals("[999,undefiniert,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(-1, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(-2, 999);
		Assert.assertEquals("[undefiniert,1,2,3,999,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(-3, 999);
		Assert.assertEquals("[undefiniert,1,2,999,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(-4, 999);
		Assert.assertEquals("[undefiniert,1,999,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(-5, 999);
		Assert.assertEquals("[undefiniert,999,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(-6, 999);
		Assert.assertEquals("[999,undefiniert,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.anhaengen(-150, 999);
		Assert.assertEquals("[999,undefiniert,1,2,3,4]", feld.toString());
	}

	/**
	 * testet das Löschen eines Elements über den Containerindex.
	 */
	@Test
	public void testLoeschenIndex() {
		Feld feld = new Feld(testFeld);
		feld.loeschen(1);
		Assert.assertEquals("[1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(2);
		Assert.assertEquals("[undefiniert,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(3);
		Assert.assertEquals("[undefiniert,1,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(4);
		Assert.assertEquals("[undefiniert,1,2,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(5);
		Assert.assertEquals("[undefiniert,1,2,3]", feld.toString());

		feld = new Feld(testFeld);
		Assert.assertTrue(feld.loeschen(6) instanceof ContainerZugriffsFehler);

		feld = new Feld(testFeld);
		Assert.assertTrue(feld.loeschen(150) instanceof ContainerZugriffsFehler);

		feld = new Feld(testFeld);
		Assert.assertTrue(feld.loeschen(0) instanceof ContainerZugriffsFehler);

		feld = new Feld(testFeld);
		feld.loeschen(-1);
		Assert.assertEquals("[undefiniert,1,2,3]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(-2);
		Assert.assertEquals("[undefiniert,1,2,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(-3);
		Assert.assertEquals("[undefiniert,1,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(-4);
		Assert.assertEquals("[undefiniert,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.loeschen(-5);
		Assert.assertEquals("[1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		Assert.assertTrue(feld.loeschen(-6) instanceof ContainerZugriffsFehler);

		feld = new Feld(testFeld);
		Assert.assertTrue(feld.loeschen(-150) instanceof ContainerZugriffsFehler);
	}

	/**
	 * testet das Löschen eines Elements über einen Schlüssel.
	 */
	@Test
	public void testElementLoeschenString() {
		try {
			testFeld.loeschen("Heinz");
			Assert.fail("Löschen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel gelöscht werden
		}
		try {
			testFeld.loeschen("1");
			Assert.fail("Löschen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel eingefügt werden
		}
		try {
			testFeld.loeschen("");
			Assert.fail("Löschen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel eingefügt werden
		}
		try {
			testFeld.loeschen(null);
			Assert.fail("Löschen per Schlüssel möglich");
		} catch (final ArgumentFehler e) {
			// in ein Feld kann nicht per Schlüssel eingefügt werden
		}
	}

	/**
	 * testet das Auslesen eines Elements über einen Index.
	 */
	@Test
	public void testGetContainerElement() {
		Assert.assertEquals(NichtWert.NICHTWERT, testFeld.getContainerElement(1));
		Assert.assertEquals(1, ((Number) testFeld.getContainerElement(2)).intValue());
		Assert.assertEquals(2, ((Number) testFeld.getContainerElement(3)).intValue());
		Assert.assertEquals(3, ((Number) testFeld.getContainerElement(4)).intValue());
		Assert.assertEquals(4, ((Number) testFeld.getContainerElement(5)).intValue());
		Assert.assertEquals(4, ((Number) testFeld.getContainerElement(-1)).intValue());
		Assert.assertEquals(3, ((Number) testFeld.getContainerElement(-2)).intValue());
		Assert.assertEquals(2, ((Number) testFeld.getContainerElement(-3)).intValue());
		Assert.assertEquals(1, ((Number) testFeld.getContainerElement(-4)).intValue());
		Assert.assertEquals(NichtWert.NICHTWERT, testFeld.getContainerElement(-5));

		final int[] invalidIdx = new int[] { 0, 6, -6, 150, -150 };

		for (final int idx : invalidIdx) {
			Assert.assertTrue(testFeld.getContainerElement(idx) instanceof ContainerZugriffsFehler);
		}
	}

	/**
	 * testet das Setzen eines Elements über den Containerindex.
	 */
	@Test
	public void testSetContainerElement() {
		Feld feld = new Feld(testFeld);
		feld.setContainerElement(1, 999);
		Assert.assertEquals("[999,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(2, 999);
		Assert.assertEquals("[undefiniert,999,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(3, 999);
		Assert.assertEquals("[undefiniert,1,999,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(4, 999);
		Assert.assertEquals("[undefiniert,1,2,999,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(5, 999);
		Assert.assertEquals("[undefiniert,1,2,3,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(6, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(8, 999);
		Assert.assertEquals("[undefiniert,1,2,3,4,undefiniert,undefiniert,999]", feld
				.toString());

		feld = new Feld(testFeld);
		try {
			feld.setContainerElement(0, 999);
			Assert.fail("Schreiben mit ungültigem Index möglich");
		} catch (final ContainerZugriffsFehler e) {
			// Schreiben korrekt abgelehnt
		}

		feld = new Feld(testFeld);
		feld.setContainerElement(-1, 999);
		Assert.assertEquals("[undefiniert,1,2,3,999]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(-2, 999);
		Assert.assertEquals("[undefiniert,1,2,999,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(-3, 999);
		Assert.assertEquals("[undefiniert,1,999,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(-4, 999);
		Assert.assertEquals("[undefiniert,999,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		feld.setContainerElement(-5, 999);
		Assert.assertEquals("[999,1,2,3,4]", feld.toString());

		feld = new Feld(testFeld);
		try {
			feld.setContainerElement(-6, 999);
			Assert.fail("Schreiben mit ungültigem Index möglich");
		} catch (final ContainerZugriffsFehler e) {
			// Schreiben korrekt abgelehnt
		}

		feld = new Feld(testFeld);
		try {
			feld.setContainerElement(-150, 999);
			Assert.fail("Schreiben mit ungültigem Index möglich");
		} catch (final ContainerZugriffsFehler e) {
			// Schreiben korrekt abgelehnt
		}

	}

	/**
	 * testet die Ausgabe eines Containers als Zeichenkette.
	 */
	@Test
	public void testToString() {
		Assert.assertEquals("[undefiniert,1,2,3,4]", testFeld.toString());
	}
}
