/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.zeit;

import de.bsvrz.ibv.uda.interpreter.Kopierbar;

/**
 * Die Klasse beschreibt eine Zeitdauer in Millisekunden. Letztendlich wird der
 * Inhalt einer Instanz durch einen Longwert repräsentiert. Die Kapselung in ein
 * spezielles Klassenobjekt wird benötigt, um Typsicherheit beim Einsatz von
 * Operatoren in Uda-Skript zu erreichen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZeitDauer implements ZeitWert, Kopierbar {

	/**
	 * die Zeitdauer in Millisekunden, ein negativer Wert ist zulässig.
	 */
	private long dauer;

	/**
	 * Konstruktor mit Angabe der enthaltenen Zeitdauer in Millisekunden.
	 *
	 * @param dauer
	 *            die Zeitdauer in Millisekunden
	 */
	public ZeitDauer(final long dauer) {
		this.dauer = dauer;
	}

	/**
	 * {@inheritDoc}.<br>
	 *
	 * @see de.bsvrz.ibv.uda.interpreter.Kopierbar#erzeugeKopie()
	 */
	@Override
	public ZeitDauer erzeugeKopie() {
		return new ZeitDauer(dauer);
	}

	/**
	 * liefert die Zeitdauer in Millisekunden, die durch die Instanz dieser
	 * Klasse repräsentiert wird.
	 *
	 * @return dauer der Wert
	 * @see #setMilliSekunden(long)
	 */
	@Override
	public long getMilliSekunden() {
		return dauer;
	}

	/**
	 * setzt den Wert der Zeitdauer auf den in Millisekunden angegebenen Wert.
	 *
	 * @param wert
	 *            der Wert
	 * @see #getMilliSekunden()
	 */
	@Override
	public void setMilliSekunden(final long wert) {
		dauer = wert;
	}

	/**
	 * gibt den Wert der Zeitdauer in Millisekunden als Zeichenkette aus.<br>
	 * {@inheritDoc}.
	 *
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return dauer + " ms";
	}
}
