/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.zeit;

import org.junit.Assert;
import org.junit.Test;

/**
 * Testklasse zum Test der Klasse: {@link ZeitDauer}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZeitDauerTest {

	/**
	 * Objekt mit der maximal möglichen Zeitdauer.
	 */
	private final ZeitDauer maximaleZeitDauer = new ZeitDauer(Long.MAX_VALUE);

	/**
	 * Objekt mit einer positiven Zeitdauer.
	 */
	private final ZeitDauer positiveZeitDauer = new ZeitDauer(3600L * 1000L);

	/**
	 * Objekt mit einer Zeitdauer == 0.
	 */
	private final ZeitDauer nullZeitDauer = new ZeitDauer(0L);

	/**
	 * Objekt mit einer negativen Zeitdauer.
	 */
	private final ZeitDauer negativeZeitDauer = new ZeitDauer(-3600L * 1000L);

	/**
	 * Objekt mit der minimal möglichen Zeitdauer.
	 */
	private final ZeitDauer minimaleZeitDauer = new ZeitDauer(Long.MIN_VALUE);

	/**
	 * Auslesen der Zeit in Millisekunden.
	 */
	@Test
	public void testGetMilliSekunden() {
		Assert.assertEquals(Long.MAX_VALUE, maximaleZeitDauer.getMilliSekunden());
		Assert.assertEquals(3600000L, positiveZeitDauer.getMilliSekunden());
		Assert.assertEquals(0L, nullZeitDauer.getMilliSekunden());
		Assert.assertEquals(-3600000L, negativeZeitDauer.getMilliSekunden());
		Assert.assertEquals(Long.MIN_VALUE, minimaleZeitDauer.getMilliSekunden());
	}

	/**
	 * Umwandlung der Zeitdauer in eine Zeichenkette.
	 */
	@Test
	public void testToString() {
		Assert.assertEquals("9223372036854775807 ms", maximaleZeitDauer.toString());
		Assert.assertEquals("3600000 ms", positiveZeitDauer.toString());
		Assert.assertEquals("0 ms", nullZeitDauer.toString());
		Assert.assertEquals("-3600000 ms", negativeZeitDauer.toString());
		Assert.assertEquals("-9223372036854775808 ms", minimaleZeitDauer.toString());
	}

	/**
	 * setzen der Zeitdauer über den Millisekundenwert.
	 */
	@Test
	public void testSetMilliSekunden() {
		final ZeitDauer zd = new ZeitDauer(0L);

		zd.setMilliSekunden(Long.MAX_VALUE);
		Assert.assertEquals(Long.MAX_VALUE, zd.getMilliSekunden());

		zd.setMilliSekunden(3600L * 1000L);
		Assert.assertEquals(3600000L, zd.getMilliSekunden());

		zd.setMilliSekunden(0L);
		Assert.assertEquals(0L, zd.getMilliSekunden());

		zd.setMilliSekunden(-3600L * 1000L);
		Assert.assertEquals(-3600000L, zd.getMilliSekunden());

		zd.setMilliSekunden(Long.MIN_VALUE);
		Assert.assertEquals(Long.MIN_VALUE, zd.getMilliSekunden());
	}
}
