/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.zeit;

import java.util.List;

import de.bsvrz.ibv.uda.interpreter.AbstractUdaHandler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.HandlerValidation;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;
import de.bsvrz.sys.funclib.bitctrl.interpreter.logik.LogischerWert;

/**
 * Handler für Operationen mit Zeitwerten.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZeitstempelHandler extends AbstractUdaHandler {

	/**
	 * liefert die Liste der unterstützten Operatoren.<br>
	 * {@inheritDoc}.
	 *
	 * @return die Liste der unterstützten Operatoren
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#getHandledOperators()
	 */
	@Override
	public Operator[] getHandledOperators() {
		return ZeitstempelOperatoren.OPERATOREN;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#validiereHandler(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public HandlerValidation validiereHandler(final Operator operator, final List<? extends Object> operanden) {
		boolean anzahlOk = false;
		boolean typOk = false;

		if (ZeitstempelOperatoren.KONSTRUKTOR.equals(operator)) {
			if ((operanden.size() == 0) || ((operanden.size() == 1) && (operanden.get(0) instanceof String))) {
				anzahlOk = true;
				typOk = true;
			}
		} else if (ZeitstempelOperatoren.ZEITBEREICH_KONSTRUKTOR.equals(operator)) {
			if ((operanden.size() == 2) || (operanden.size() == 3)) {
				anzahlOk = true;
				Object op1 = null;
				Object op2 = null;
				Object op3 = null;
				if (operanden.size() > 0) {
					op1 = operanden.get(0);
				}
				if (operanden.size() > 1) {
					op2 = operanden.get(1);
				}
				if (operanden.size() > 2) {
					op3 = operanden.get(2);
				}

				if (op1 instanceof Zeitstempel) {
					if ((op2 instanceof Zeitstempel) || (op2 instanceof ZeitDauer)) {
						if ((op3 == null) || (op3 instanceof LogischerWert)) {
							typOk = true;
						}
					}
				}
			}
		} else if (ZeitstempelOperatoren.FORMAT.equals(operator)) {
			if ((operanden.size() > 0) && (operanden.size() <= 2)) {
				anzahlOk = true;
				if (operanden.get(0) instanceof Zeitstempel) {
					if (operanden.size() == 2) {
						if (operanden.get(1) instanceof String) {
							typOk = true;
						}
					} else {
						typOk = true;
					}
				}
			}
		} else if (ZeitstempelOperatoren.PLUS.equals(operator)) {
			if (operanden.size() == 2) {
				anzahlOk = true;
				final Object operand1 = operanden.get(0);
				final Object operand2 = operanden.get(1);
				if ((operand1 instanceof Zeitstempel) && (operand2 instanceof ZeitDauer)) {
					typOk = true;
				} else if ((operand1 instanceof ZeitDauer)
						&& ((operand2 instanceof ZeitDauer) || (operand2 instanceof Zeitstempel))) {
					typOk = true;
				}
			}
		} else if (ZeitstempelOperatoren.MINUS.equals(operator)) {
			if (operanden.size() == 2) {
				anzahlOk = true;
				final Object operand1 = operanden.get(0);
				final Object operand2 = operanden.get(1);
				if ((operand2 instanceof ZeitDauer)
						&& ((operand1 instanceof ZeitDauer) || (operand1 instanceof Zeitstempel))) {
					typOk = true;
				} else if ((operand1 instanceof Zeitstempel) && (operand2 instanceof Zeitstempel)) {
					typOk = true;
				}
			}
		}

		return new HandlerValidation(anzahlOk, typOk);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.sys.funclib.bitctrl.interpreter.Handler#perform(de.bsvrz.sys.funclib.bitctrl.interpreter.Operator,
	 *      java.util.List)
	 */
	@Override
	public Object perform(final Operator operator, final List<Object> operanden) {
		erzeugeHandlerFehler(operator, operanden);

		Object result = null;

		if (operator.equals(ZeitstempelOperatoren.KONSTRUKTOR)) {
			String zeitAngabe = null;
			if (operanden.size() > 0) {
				zeitAngabe = (String) operanden.get(0);
			}
			result = erzeugeZeitstempel(zeitAngabe);
		} else if (operator.equals(ZeitstempelOperatoren.ZEITBEREICH_KONSTRUKTOR)) {
			final Zeitstempel start = (Zeitstempel) operanden.get(0);
			final Object ende = operanden.get(1);
			boolean inklusive = false;
			if (operanden.size() > 2) {
				inklusive = ((LogischerWert) operanden.get(2)).getBoolWert();
			}

			result = new ZeitBereich(start, ende, inklusive);
		} else if (operator.equals(ZeitstempelOperatoren.FORMAT)) {
			final Zeitstempel zeitstempel = (Zeitstempel) operanden.get(0);
			String format = null;
			if (operanden.size() == 2) {
				format = (String) operanden.get(1);
			}
			result = formatiereZeitStempel(zeitstempel, format);
		} else if (operator.equals(ZeitstempelOperatoren.PLUS)) {
			result = addiereZeitWerte((ZeitWert) operanden.get(0), (ZeitWert) operanden.get(1));
		} else if (operator.equals(ZeitstempelOperatoren.MINUS)) {
			result = subtrahiereZeitWerte((ZeitWert) operanden.get(0), (ZeitWert) operanden.get(1));
		}
		return result;
	}

	/**
	 * Die Funktion ermittelt die Differenz zwischen zwei Zeitwerten. Die
	 * Differenz zwischen zwei Seitstempeln bzw. zwei Zeitdauern ergibt eine
	 * Zeitdauer. Die Differenz zwischen einem Zeitstempel und einer Zeitdauer
	 * ergibt einen Zeitstempel.
	 *
	 * @param wert1
	 *            der Minuend
	 * @param wert2
	 *            der Subtrahend
	 * @return die Differenz
	 */
	private Object subtrahiereZeitWerte(final ZeitWert wert1, final ZeitWert wert2) {
		final long differenz = wert1.getMilliSekunden() - wert2.getMilliSekunden();
		if (wert1.getClass().equals(wert2.getClass())) {
			return new ZeitDauer(differenz);
		}
		return new Zeitstempel(differenz);
	}

	/**
	 * Die Funktion bildet die SUmme zweier Zeitwerte. Die Summe zweier
	 * Zeitdauern ergibt eine Zeitdauer. Die Summe von Zeitdauer und Zeitstempel
	 * ergibt einen Zeitstempel.
	 *
	 * @param wert1
	 *            der erste Summand
	 * @param wert2
	 *            der zweite Summand
	 * @return die Summe
	 */
	private Object addiereZeitWerte(final ZeitWert wert1, final ZeitWert wert2) {
		final long summe = wert1.getMilliSekunden() + wert2.getMilliSekunden();
		if ((wert1 instanceof ZeitDauer) && (wert2 instanceof ZeitDauer)) {
			return new ZeitDauer(summe);
		}
		return new Zeitstempel(summe);
	}

	/**
	 * Die Funktion wandelt einen übergebenen Zeitstempel unter Berücksichtigung
	 * des übergebenen Formatstrings in eine Zeichenkette um.
	 *
	 * @param zeitstempel
	 *            der Zeitstempel
	 * @param format
	 *            der Formatstring
	 * @return die Zeichenkette, die den Zeitstempel darstellt.
	 */
	private Object formatiereZeitStempel(final Zeitstempel zeitstempel, final String format) {
		return zeitstempel.format(format);
	}

	/**
	 * die Funktion erzeugt einen Zeitstempel, der mit der üübergebenen
	 * Zietangabe initialisiert wird.
	 *
	 * @param zeitAngabe
	 *            die Zeitangabe
	 * @return der Zeitstempel
	 */
	private Object erzeugeZeitstempel(final String zeitAngabe) {
		return new Zeitstempel(zeitAngabe);
	}
}
