/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.uda.data;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;

/**
 * die abstrakte Realisierung eines UdaModuls.<br>
 * Instanzen dieser Klasse repräsentieren ein UdaModul, das durch ein
 * Systemobjekt innerhalb der Datenverteilerkonfiguration dargestellt sind.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public abstract class Modul {
	/**
	 * das Objekt, das das Modul innerhalb der Datenverteilerkonfiguration
	 * darstellt.
	 */
	private final ConfigurationObject modulObjekt;

	/**
	 * die verwendete Datenverteilerverbindung.
	 */
	private ClientDavInterface verbindung;

	/**
	 * Konstruktor.
	 *
	 * @param verbindung
	 *            die verwendete Datenverteilerverbindung
	 * @param modulObjekt
	 *            das Objekt innerhalb der Datenverteilerkonfiguration
	 */
	public Modul(final ClientDavInterface verbindung, final ConfigurationObject modulObjekt) {
		this.verbindung = verbindung;
		this.modulObjekt = modulObjekt;
	}

	/**
	 * liefert das Datenverteiler-Systemobjekt.
	 *
	 * @return das Objekt
	 */
	public ConfigurationObject getModulObjekt() {
		return modulObjekt;
	}

	/**
	 * liefert die verwendete Datenverteilerverbindung.
	 *
	 * @return die Verbindung
	 */
	public ClientDavInterface getVerbindung() {
		return verbindung;
	}

	/**
	 * setzt die Verbindung, über den das Modul mit dem Datenverteiler
	 * kommunizieren sollen.
	 *
	 * @param verbindung
	 *            die Verbindung
	 */
	public void setVerbindung(final ClientDavInterface verbindung) {
		this.verbindung = verbindung;
	}

	/**
	 * liefert ein Skript des Moduls.<br>
	 * Aus der Menge der für das Modul angelegten Skripte wird das mit der
	 * übergebenen PID ermittelt.
	 *
	 * @param pid
	 *            die PID des gesuchten Skripts
	 * @return das Skript
	 */
	public abstract Skript getSkript(String pid);
}
