/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.verwaltung;

import java.security.InvalidKeyException;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.interfaces.RSAPublicKey;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

/**
 * das Schlüsselpaar für die gesicherte Übertragung von Passworten zwischem dem
 * Client der "Umfassenden Datenanalyse" und der "Umfassenden Datananalyse"
 * selbst.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaSchluessel {

	/**
	 * das Schlüsselpaar für die Verschlüsselung.
	 */
	private KeyPair keys;

	/**
	 * der Code für die Verschlüsselung.
	 */
	private Cipher cipher;

	/**
	 * Konstruktor.
	 *
	 * @throws UdaServerException
	 *             der Schlüssel konnte nicht erzeugt werden.
	 */
	public UdaSchluessel() throws UdaServerException {
		try {
			final KeyPairGenerator generator = KeyPairGenerator.getInstance("RSA");
			generator.initialize(1024);
			keys = generator.generateKeyPair();
			cipher = Cipher.getInstance("RSA");

		} catch (final NoSuchAlgorithmException e) {
			throw new UdaServerException("Passwort-Verschlüsselung konnte nicht initialisiert werden",
					e);
		} catch (final NoSuchPaddingException e) {
			throw new UdaServerException("Passwort-Verschlüsselung konnte nicht initialisiert werden",
					e);
		}
	}

	/**
	 * übersetzt ein Byte-Feld mit einem verschlüsseltenPasswort in Klartext.
	 *
	 * @param daten
	 *            die verschlüsselten Daten
	 * @return den entschlüsselten Text
	 * @throws UdaServerException
	 *             die Übersetzung war nicht erfolgreich
	 */
	public String decrypt(final byte[] daten) throws UdaServerException {
		try {
			cipher.init(Cipher.DECRYPT_MODE, keys.getPrivate());
			return new String(cipher.doFinal(daten));
		} catch (final InvalidKeyException e) {
			throw new UdaServerException("Passwort konnte nicht entschlüsselt werden", e);
		} catch (final IllegalBlockSizeException e) {
			throw new UdaServerException("Passwort konnte nicht entschlüsselt werden", e);
		} catch (final BadPaddingException e) {
			throw new UdaServerException("Passwort konnte nicht entschlüsselt werden", e);
		}
	}

	/**
	 * liefert den öffentlichen Schlüssel.
	 *
	 * @return den Schlüssel
	 */
	public RSAPublicKey getPublic() {
		return (RSAPublicKey) keys.getPublic();
	}
}
